#include "StdAfx.h"
#include "GameTalkAPIGlobal.h"
#include "WASListBuddyGroup.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"


CWASListBuddyGroup::CWASListBuddyGroup(CWASActor *pActor) : 
	CWASCommand(pActor),
	m_nDefaultGroupID(1)
{
}


CWASListBuddyGroup::~CWASListBuddyGroup(void)
{
}


void CWASListBuddyGroup::ExecuteAsync()
{
	m_listGroupPrivate.clear();
	m_listGroup.clear();

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;

	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/buddy/ListBuddyGroup.php");

	m_httpFacade.session().SetURL( W2U8(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}



bool CWASListBuddyGroup::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	//
	//	[XML FORMAT]
	/*
		<?xml version="1.0" encoding="UTF-8" ?>
		<nphone>
			<nodes num='#ڿ'>
				<node grprid='#ڿ'  grpname='#ڿ' />
			</nodes>
		</nphone>
	*/
	if(pDom == NULL)
		return false;

	nfc::IIkXMLNode* pNode = pDom->GetdocumentElement()->selectSingleNode("nodes");
	if (pNode == NULL)
	{
		m_strWasErrMsg += _T("can not find \"<nodes>\" in group list.");
		return false;
	}

	nfc::IIkXMLNodeList *pList = pNode->selectNodes("node");		
	nfc::IIkXMLAttribute* pAttr;

	USES_UTF8_CONVERSION;

	m_listGroupPrivate.clear();

	int nFirstGroupID = -1;
	for (int i = 0; i < pList->Getlength(); i++)
	{	
		pNode = pList->Getitem(i);
		GT_ASSERT(pNode);

		if (NULL == (pAttr = pNode->getAttribute("grpid")))
			continue;	// ignore this item

		P_GT_BUDDY_GROUP_DATA group;

		group.nID = atoi(pAttr->Gettext().c_str());

		if (NULL == (pAttr = pNode->getAttribute("grpname")))
		{
			//continue;	// ignore this item
			// ignore  ,, ӽ ̸ .

			group.strName = _T("<no name>");
		}
		else
		{
			group.strName = U82W(pAttr->Gettext().c_str());
		}

		if (NULL == (pAttr = pNode->getAttribute("sortorder")))
		{
			
		}
		group.nSortOrder = atoi(pAttr->Gettext().c_str());

		m_listGroupPrivate.push_back(group);

		nFirstGroupID = group.nID;
		
		// ׷ ⺻׷ ̸   ̵ ⺻ ׷ ID ´.
		if (group.strName == GT_GLOBAL->GetEnvironment().strDefaultBuddyGroupName)
		{
			m_nDefaultGroupID = group.nID;
		}
	}

	// only one group'id in list is must be default group.
	if(pList->Getlength() == 1 && nFirstGroupID >= 0)
	{
		m_nDefaultGroupID = nFirstGroupID;
	}
	
	return true;
}


// override in drived class
void CWASListBuddyGroup::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t observers;
	m_pActor->GetObserversEx(&observers);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t::iterator it = observers.begin();

	// WSTRING LPCWSTR ȯؼ 迭  ־!!!
	for(size_t i = 0; i < m_listGroupPrivate.size(); i++)
	{
		GT_BUDDY_GROUP_DATA sTemp;
		m_listGroupPrivate[i].Convert(sTemp);
		m_listGroup.push_back(sTemp);
	}

	GT_BUDDY_GROUP_DATA_LIST sList;
	sList.nSize = m_listGroup.size();
	sList.pList = sList.nSize > 0 ? &m_listGroup[0] : NULL;
	
	for (; it != observers.end(); ++it)
	{
		IGTWASActorEventPrivate *pObserver = static_cast<IGTWASActorEventPrivate *>(*it);
		if(pObserver->OnGTWASActor_ResponseBuddyGroupList(m_pActor, GetID(), m_eError, m_nDefaultGroupID, sList) == false)
			break;
	}
	
	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASListBuddyGroup::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t observers;
	m_pActor->GetObserversEx(&observers);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t::iterator it = observers.begin();

	// WSTRING LPCWSTR ȯؼ 迭  ־!!!
	for(size_t i = 0; i < m_listGroupPrivate.size(); i++)
	{
		GT_BUDDY_GROUP_DATA sTemp;
		m_listGroupPrivate[i].Convert(sTemp);
		m_listGroup.push_back(sTemp);
	}

	GT_BUDDY_GROUP_DATA_LIST sList;
	sList.nSize = m_listGroup.size();
	sList.pList = sList.nSize > 0 ? &m_listGroup[0] : NULL;
	
	for (; it != observers.end(); ++it)
	{
		IGTWASActorEventPrivate *pObserver = static_cast<IGTWASActorEventPrivate *>(*it);
		if(pObserver->OnGTWASActor_ResponseBuddyGroupList(m_pActor, GetID(), m_eError, m_nDefaultGroupID, sList) == false)
			break;
	}

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}
